package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.biz.dal.entity.SysSuggestion;
import com.yonge.cooleshow.biz.dal.mapper.SysSuggestionMapper;
import com.yonge.cooleshow.biz.dal.service.SysSuggestionService;
import com.yonge.cooleshow.biz.dal.wrapper.SysSuggestionWrapper;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * 平台建议表
 * 2023-12-20 11:27:10
 */
@Slf4j
@Service
public class SysSuggestionServiceImpl extends ServiceImpl<SysSuggestionMapper, SysSuggestion> implements SysSuggestionService {

	/**
	 * 查询详情
	 * @param id 详情ID
	 * @return SysSuggestion
	 */
	@Override
	public SysSuggestion detail(Long id) {

		return baseMapper.selectById(id);
	}

	/**
	 * 分页查询
	 * @param page IPage<SysSuggestion>
	 * @param query SysSuggestionWrapper.SysSuggestionQuery
	 * @return IPage<SysSuggestion>
	 */
	@Override
	public IPage<SysSuggestionWrapper.SysSuggestion> selectPage(IPage<SysSuggestionWrapper.SysSuggestion> page, SysSuggestionWrapper.SysSuggestionQuery query) {

		return page.setRecords(baseMapper.selectPage(page, query));
	}

	/**
	 * 添加
	 * @param sysSuggestion SysSuggestionWrapper.SysSuggestion
	 * @return Boolean
	 */
	@Override
	public Boolean add(SysSuggestionWrapper.SysSuggestion sysSuggestion) {

		return this.save(JSON.parseObject(sysSuggestion.jsonString(), SysSuggestion.class));
	}

	/**
	 * 更新
	 * @param sysSuggestion SysSuggestionWrapper.SysSuggestion
	 * @return Boolean
	 */
	@Override
	public Boolean update(SysSuggestionWrapper.SysSuggestion sysSuggestion){

		return this.updateById(JSON.parseObject(sysSuggestion.jsonString(), SysSuggestion.class));
	}

	/**
	 * 处理反馈意见
	 * @param handleSuggestion handleSuggestion
	 * @return true/false
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public Boolean handleSuggestion(SysSuggestionWrapper.HandleSuggestion handleSuggestion) {
		Long id = handleSuggestion.getId();
		SysSuggestion suggestion = this.getById(id);
		if (suggestion == null) {
			throw new BizException("反馈意见不存在");
		}
		Boolean handleStatus = suggestion.getHandleStatus();
		if (Boolean.TRUE.equals(handleStatus)) {
			throw new BizException("反馈意见已经处理");
		}
		this.lambdaUpdate()
				.set(SysSuggestion::getHandleStatus, true)
				.set(SysSuggestion::getHandleTime, new Date())
				.set(SysSuggestion::getFeedback, handleSuggestion.getFeedback())
				.set(SysSuggestion::getFeedbackContent, handleSuggestion.getFeedbackContent())
				.set(SysSuggestion::getHandleBy,handleSuggestion.getHandleBy())
				.eq(SysSuggestion::getId, handleSuggestion.getId())
				.eq(SysSuggestion::getHandleStatus, false)
				.update();
		return true;
	}
}