package com.yonge.cooleshow.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.yonge.cooleshow.biz.dal.enums.SuggestionType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.Optional;

/**
 * 平台建议表
 * 2023-12-20 11:27:10
 */
@ApiModel(value = "SysSuggestionWrapper对象", description = "平台建议表查询对象")
public class SysSuggestionWrapper {

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SysSuggestionQuery-平台建议表")
    public static class SysSuggestionQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("关键字匹配")
        private String keyword;

        @ApiModelProperty(value = "建议类型")
        private SuggestionType type;

        private String startTime;

        private String endTime;

        @ApiModelProperty(value = "客户端类型")
        private String clientType;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SysSuggestionQuery from(String json) {
            return JSON.parseObject(json, SysSuggestionQuery.class);
        }
    }

    @Data
    @ApiModel(" SysSuggestion-平台建议表")
    public static class SysSuggestion {

        @ApiModelProperty("编号")
        private Long id;

        @ApiModelProperty("联系方式")
        private String mobileNo;

        @ApiModelProperty("标题")
        private String title;

        @ApiModelProperty("内容")
        private String content;

        @ApiModelProperty("链接")
        private String url;

        @ApiModelProperty("用户编号")
        private Long userId;

        @ApiModelProperty("提交时间")
        private Date createTime;

        @ApiModelProperty("客户端类型")
        private String clientId;

        @ApiModelProperty("建议类型")
        private SuggestionType type;

        @ApiModelProperty("客户端类型")
        private String clientType;

        @ApiModelProperty("建议类型ID")
        private Long suggestionTypeId;

        @ApiModelProperty("客户端信息")
        private String userAgent;

        @ApiModelProperty("处理状态：true:已处理，false:待处理")
        private Boolean handleStatus;

        @ApiModelProperty("处理时间")
        private Date handleTime;

        @ApiModelProperty("处理人")
        private Long handleBy;

        @ApiModelProperty("是否反馈用户,true:是，false:否")
        private Boolean feedback;

        @ApiModelProperty("反馈内容")
        private String feedbackContent;

//        @ApiModelProperty(value = "用户名")
//        private String username;

        @ApiModelProperty(value = "处理人姓名")
        private String handleName;

        @ApiModelProperty("建议类型名称")
        private String suggestionTypeName;

        @ApiModelProperty("用户名")
        private String nickname;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SysSuggestion from(String json) {
            return JSON.parseObject(json, SysSuggestion.class);
        }
    }

    @ApiModel("处理反馈意见模型")
    @Data
    public static class HandleSuggestion {

        @ApiModelProperty("id")
        @NotNull
        private Long id;

        @ApiModelProperty("是否反馈用户,true:是，false:否")
        @NotNull
        private Boolean feedback;

        @ApiModelProperty("反馈内容")
        @NotNull
        private String feedbackContent;

        @ApiModelProperty(value = "处理人", hidden = true)
        private Long handleBy;

    }

}